"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("./constants");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const getDelayedPromise = continuation => {
  let fastEventLoop = false;

  try {
    fastEventLoop = +process.version.substring(1).split('.')[0] >= 12;
  } catch (e) {}

  let [promiseResolve, promiseReject, promise] = [null, null, null];

  if (!fastEventLoop) {
    promise = new Promise((resolve, reject) => {
      promiseResolve = resolve;
      promiseReject = reject;
    });
  } else {
    const internalPromise = Symbol();
    promise = {
      [internalPromise]: {
        thenFunctions: [],
        catchFunctions: [],
        state: null,
        result: null
      },
      then: next => {
        if (promise[internalPromise].state == null) {
          promise[internalPromise].thenFunctions.push(next);
        } else if (promise[internalPromise].state === true) {
          next(promise[internalPromise].result);
        }
      },
      catch: next => {
        if (promise[internalPromise].state == null) {
          promise[internalPromise].catchFunctions.push(next);
        } else if (promise[internalPromise].state === false) {
          next(promise[internalPromise].result);
        }
      }
    };

    promiseResolve = result => {
      if (promise[internalPromise].state == null) {
        promise[internalPromise].state = true;
        promise[internalPromise].result = result;

        for (const next of promise[internalPromise].thenFunctions) {
          next(result);
        }

        promise[internalPromise].thenFunctions.length = 0;
      }
    };

    promiseReject = result => {
      if (promise[internalPromise].state == null) {
        promise[internalPromise].state = false;
        promise[internalPromise].result = result;

        for (const next of promise[internalPromise].catchFunctions) {
          next(result);
        }

        promise[internalPromise].catchFunctions.length = 0;
      }
    };
  }

  const promiseThen = promise.then.bind(promise);
  const promiseCatch = promise.catch.bind(promise);
  promise.then = continuation.bind(null, promiseThen);
  promise.catch = continuation.bind(null, promiseCatch);
  return {
    promiseResolve,
    promiseReject,
    promise
  };
};

const getInitPromise = internalPool => new Promise(initResolve => {
  internalPool.initResolve = async (continuation, ...args) => {
    initResolve();
    return await continuation(...args);
  };
});

const givenEvents = (dependencies, events) => {
  const {
    readModel,
    as,
    init,
    saga,
    properties
  } = dependencies;
  const internalPool = Object.create(null);
  const initPromise = getInitPromise(internalPool);
  const {
    promiseResolve,
    promiseReject,
    promise
  } = getDelayedPromise(internalPool.initResolve);

  const pool = _objectSpread({}, dependencies, {
    promise
  });

  promise[_constants.symbol] = internalPool;
  promise[_constants.symbol].resolve = promiseResolve;
  promise[_constants.symbol].reject = promiseReject;
  promise[_constants.symbol].initPromise = initPromise;
  promise[_constants.symbol].events = events;
  promise[_constants.symbol].phase = _constants.Phases.GIVEN_EVENTS;
  promise[_constants.symbol].properties = {
    RESOLVE_SIDE_EFFECTS_START_TIMESTAMP: 0
  };
  promise.readModel = readModel.bind(null, pool);
  promise.as = as.bind(null, pool);
  promise.saga = saga.bind(null, pool);
  promise.properties = properties.bind(null, pool);
  initPromise.then(init.bind(null, pool));
  return promise;
};

var _default = givenEvents;
exports.default = _default;
//# sourceMappingURL=given-events.js.map